import os
import subprocess


def install(src, dest, fileprops=None, sudo=False):
    """Install a file from src to dest. Dest can be a complete filename
    or a target directory. fileprops is a dict with 'owner' (username of owner)
    and mode (octal string) as keys, the defaults are 'ubuntu' and '400'

    When owner is passed or when access requires it sudo can be set to True and
    sudo will be used to install the file.
    """
    if not fileprops:
        fileprops = {}
    mode = fileprops.get('mode', '400')
    owner = fileprops.get('owner')
    cmd = ['install']

    if not os.path.exists(src):
        raise OSError(src)

    if not os.path.exists(dest) and not os.path.exists(os.path.dirname(dest)):
        # create all but the last component as path
        cmd.append('-D')

    if mode:
        cmd.extend(['-m', mode])

    if owner:
        cmd.extend(['-o', owner])

    if sudo:
        cmd.insert(0, 'sudo')
    cmd.extend([src, dest])
    subprocess.check_call(cmd)
