#!/usr/bin/env python
# vim: et sta sts ai ts=4 sw=4:
import os
import sys

from charmhelpers.core import hookenv
from charmhelpers.core.hookenv import log
from charmhelpers.contrib.cloudfoundry import contexts
from charmhelpers.contrib.cloudfoundry import services
from config import (
    SERVICE_USER, JOB_DIR, PACKAGES_DIR, LOG_DIR
)

hooks = hookenv.Hooks()
fileproperties = {'owner': SERVICE_USER, 'mode': '664'}

settings_target = os.path.join(
    PACKAGES_DIR, 'cf-mysql-broker/config/settings.yml'
)
db_target = os.path.join(
    PACKAGES_DIR, 'cf-mysql-broker/config/database.yml'
)


broker_address = hookenv.unit_get('private-address').encode('utf-8')


services.register([
    {
        'service': 'cf-mysql-broker',
        'templates': [{
            'source': 'settings.yml',
            'target': settings_target,
            'file_properties': fileproperties,
            'contexts': [
                contexts.NatsContext(),
                contexts.RouterContext(),
                contexts.StaticContext(
                    {'broker_ip': broker_address}
                )
            ]
        }, {
            'source': 'database.yml',
            'target': db_target,
            'file_properties': fileproperties,
            'contexts': [contexts.MySQLContext()]
        }, {
            'source': 'cf-mysql-broker.conf',
            'target': '/etc/init/cf-mysql-broker.conf',
            'file_properties': {'owner': SERVICE_USER, 'mode': '664'},
            'contexts': [
                contexts.RouterContext(),
                contexts.StaticContext(
                    {'broker_ip': broker_address, 'log_dir': LOG_DIR}
                )
            ]
        }, {
            'source': 'unicorn.conf.rb',
            'target': os.path.join(JOB_DIR, 'config', 'unicorn.conf.rb'),
            'contexts': [
                contexts.StaticContext({'unicorn_port': '80'})
            ]
        }]
    }
], os.path.join(hookenv.charm_dir(), 'templates'))


@hooks.hook()
def start():
    pass
#    hookenv.open_port(80)
#    hookenv.open_port(443)


@hooks.hook("config-changed")
def config_changed():
    pass
#    services.reconfigure_services()
#    domain = get_domain()
#    services.reconfigure_services()
#    for rid in hookenv.relation_ids('router'):
#        hookenv.relation_set(rid, {'domain': domain})


@hooks.hook()
def stop():
    pass
#   services.stop_services()
#    hookenv.close_port(80)
#    hookenv.close_port(443)


@hooks.hook("db-admin-relation-changed")
def db_admin_relation_changed():
    services.reconfigure_services()
#    with chdir(os.path.join(PACKAGES_DIR, 'cf-mysql-broker')):
#        run(['bundle', 'exec', 'rake', 'db:setup'])


@hooks.hook("router-relation-changed")
def router_relation_changed():
    services.reconfigure_services()


@hooks.hook("nats-relation-changed")
def nats_relation_changed():
    services.reconfigure_services()


if __name__ == '__main__':
    hook_name = os.path.basename(sys.argv[0])
    log("Running {} hook".format(hook_name))
    if hookenv.relation_id():
        log("Relation {} with {}".format(
            hookenv.relation_id(), hookenv.remote_unit()))
    hooks.execute(sys.argv)
